/*
	Copyright (C) 2023 n0n3m4 (Roman Lebedev)
	All rights reserved.
	Redistribution and use is allowed only by written personal permission of n0n3m4 (Roman Lebedev).
	Any other use is prohibited
*/

/*
	This file is a part of Backcompat library for Android.
*/

#ifndef BACKCOMPAT_SPAWN_H
#define BACKCOMPAT_SPAWN_H

// This header is only required for Bionic libc
// else we just include the next one
#ifndef __ANDROID__

// In non-Bionic we just remove those functions
#define posix_spawn __backcompat_orig_posix_spawn
#define posix_spawnp __backcompat_orig_posix_spawnp

// Embedded file into libbackcompat: spawn.h

/*
 * Copyright (C) 2017 The Android Open Source Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _SPAWN_H_
#define _SPAWN_H_

#include <sys/cdefs.h>
#include <sys/types.h>
#include <sched.h>
#include <signal.h>

__BEGIN_DECLS

#define POSIX_SPAWN_RESETIDS 1
#define POSIX_SPAWN_SETPGROUP 2
#define POSIX_SPAWN_SETSIGDEF 4
#define POSIX_SPAWN_SETSIGMASK 8
#define POSIX_SPAWN_SETSCHEDPARAM 16
#define POSIX_SPAWN_SETSCHEDULER 32
#if defined(__USE_GNU)
#define POSIX_SPAWN_USEVFORK 64
#define POSIX_SPAWN_SETSID 128
#endif

typedef struct __posix_spawnattr* posix_spawnattr_t;
typedef struct __posix_spawn_file_actions* posix_spawn_file_actions_t;


#if __ANDROID_API__ >= 28
int posix_spawn(pid_t* __pid, const char* __path, const posix_spawn_file_actions_t* __actions, const posix_spawnattr_t* __attr, char* const __argv[], char* const __env[]) __INTRODUCED_IN(28);
int posix_spawnp(pid_t* __pid, const char* __file, const posix_spawn_file_actions_t* __actions, const posix_spawnattr_t* __attr, char* const __argv[], char* const __env[]) __INTRODUCED_IN(28);

int posix_spawnattr_init(posix_spawnattr_t* __attr) __INTRODUCED_IN(28);
int posix_spawnattr_destroy(posix_spawnattr_t* __attr) __INTRODUCED_IN(28);

int posix_spawnattr_setflags(posix_spawnattr_t* __attr, short __flags) __INTRODUCED_IN(28);
int posix_spawnattr_getflags(const posix_spawnattr_t* __attr, short* __flags) __INTRODUCED_IN(28);

int posix_spawnattr_setpgroup(posix_spawnattr_t* __attr, pid_t __pgroup) __INTRODUCED_IN(28);
int posix_spawnattr_getpgroup(const posix_spawnattr_t* __attr, pid_t* __pgroup) __INTRODUCED_IN(28);

int posix_spawnattr_setsigmask(posix_spawnattr_t* __attr, const sigset_t* __mask) __INTRODUCED_IN(28);
int posix_spawnattr_setsigmask64(posix_spawnattr_t* __attr, const sigset64_t* __mask) __INTRODUCED_IN(28);
int posix_spawnattr_getsigmask(const posix_spawnattr_t* __attr, sigset_t* __mask) __INTRODUCED_IN(28);
int posix_spawnattr_getsigmask64(const posix_spawnattr_t* __attr, sigset64_t* __mask) __INTRODUCED_IN(28);

int posix_spawnattr_setsigdefault(posix_spawnattr_t* __attr, const sigset_t* __mask) __INTRODUCED_IN(28);
int posix_spawnattr_setsigdefault64(posix_spawnattr_t* __attr, const sigset64_t* __mask) __INTRODUCED_IN(28);
int posix_spawnattr_getsigdefault(const posix_spawnattr_t* __attr, sigset_t* __mask) __INTRODUCED_IN(28);
int posix_spawnattr_getsigdefault64(const posix_spawnattr_t* __attr, sigset64_t* __mask) __INTRODUCED_IN(28);

int posix_spawnattr_setschedparam(posix_spawnattr_t* __attr, const struct sched_param* __param) __INTRODUCED_IN(28);
int posix_spawnattr_getschedparam(const posix_spawnattr_t* __attr, struct sched_param* __param) __INTRODUCED_IN(28);

int posix_spawnattr_setschedpolicy(posix_spawnattr_t* __attr, int __policy) __INTRODUCED_IN(28);
int posix_spawnattr_getschedpolicy(const posix_spawnattr_t* __attr, int* __policy) __INTRODUCED_IN(28);

int posix_spawn_file_actions_init(posix_spawn_file_actions_t* __actions) __INTRODUCED_IN(28);
int posix_spawn_file_actions_destroy(posix_spawn_file_actions_t* __actions) __INTRODUCED_IN(28);

int posix_spawn_file_actions_addopen(posix_spawn_file_actions_t* __actions, int __fd, const char* __path, int __flags, mode_t __mode) __INTRODUCED_IN(28);
int posix_spawn_file_actions_addclose(posix_spawn_file_actions_t* __actions, int __fd) __INTRODUCED_IN(28);
int posix_spawn_file_actions_adddup2(posix_spawn_file_actions_t* __actions, int __fd, int __new_fd) __INTRODUCED_IN(28);
#endif /* __ANDROID_API__ >= 28 */


__END_DECLS

#endif

// End of embedded file



#undef posix_spawn
#undef posix_spawnp

#else

#define posix_spawn __backcompat_orig_posix_spawn
#define posix_spawnp __backcompat_orig_posix_spawnp
#define posix_spawnattr_init __backcompat_orig_posix_spawnattr_init
#define posix_spawnattr_destroy __backcompat_orig_posix_spawnattr_destroy
#define posix_spawnattr_setflags __backcompat_orig_posix_spawnattr_setflags
#define posix_spawnattr_getflags __backcompat_orig_posix_spawnattr_getflags
#define posix_spawnattr_setpgroup __backcompat_orig_posix_spawnattr_setpgroup
#define posix_spawnattr_getpgroup __backcompat_orig_posix_spawnattr_getpgroup
#define posix_spawnattr_setsigmask __backcompat_orig_posix_spawnattr_setsigmask
#define posix_spawnattr_getsigmask __backcompat_orig_posix_spawnattr_getsigmask
#define posix_spawnattr_setsigdefault __backcompat_orig_posix_spawnattr_setsigdefault
#define posix_spawnattr_getsigdefault __backcompat_orig_posix_spawnattr_getsigdefault
#define posix_spawnattr_setschedparam __backcompat_orig_posix_spawnattr_setschedparam
#define posix_spawnattr_getschedparam __backcompat_orig_posix_spawnattr_getschedparam
#define posix_spawnattr_setschedpolicy __backcompat_orig_posix_spawnattr_setschedpolicy
#define posix_spawnattr_getschedpolicy __backcompat_orig_posix_spawnattr_getschedpolicy
#define posix_spawn_file_actions_init __backcompat_orig_posix_spawn_file_actions_init
#define posix_spawn_file_actions_destroy __backcompat_orig_posix_spawn_file_actions_destroy
#define posix_spawn_file_actions_addopen __backcompat_orig_posix_spawn_file_actions_addopen
#define posix_spawn_file_actions_addclose __backcompat_orig_posix_spawn_file_actions_addclose
#define posix_spawn_file_actions_adddup2 __backcompat_orig_posix_spawn_file_actions_adddup2

#define posix_spawnattr_setsigmask64 __backcompat_orig_posix_spawnattr_setsigmask64
#define posix_spawnattr_getsigmask64 __backcompat_orig_posix_spawnattr_getsigmask64
#define posix_spawnattr_setsigdefault64 __backcompat_orig_posix_spawnattr_setsigdefault64
#define posix_spawnattr_getsigdefault64 __backcompat_orig_posix_spawnattr_getsigdefault64

// Embedded file into libbackcompat: spawn.h

/*
 * Copyright (C) 2017 The Android Open Source Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _SPAWN_H_
#define _SPAWN_H_

#include <sys/cdefs.h>
#include <sys/types.h>
#include <sched.h>
#include <signal.h>

__BEGIN_DECLS

#define POSIX_SPAWN_RESETIDS 1
#define POSIX_SPAWN_SETPGROUP 2
#define POSIX_SPAWN_SETSIGDEF 4
#define POSIX_SPAWN_SETSIGMASK 8
#define POSIX_SPAWN_SETSCHEDPARAM 16
#define POSIX_SPAWN_SETSCHEDULER 32
#if defined(__USE_GNU)
#define POSIX_SPAWN_USEVFORK 64
#define POSIX_SPAWN_SETSID 128
#endif

typedef struct __posix_spawnattr* posix_spawnattr_t;
typedef struct __posix_spawn_file_actions* posix_spawn_file_actions_t;


#if __ANDROID_API__ >= 28
int posix_spawn(pid_t* __pid, const char* __path, const posix_spawn_file_actions_t* __actions, const posix_spawnattr_t* __attr, char* const __argv[], char* const __env[]) __INTRODUCED_IN(28);
int posix_spawnp(pid_t* __pid, const char* __file, const posix_spawn_file_actions_t* __actions, const posix_spawnattr_t* __attr, char* const __argv[], char* const __env[]) __INTRODUCED_IN(28);

int posix_spawnattr_init(posix_spawnattr_t* __attr) __INTRODUCED_IN(28);
int posix_spawnattr_destroy(posix_spawnattr_t* __attr) __INTRODUCED_IN(28);

int posix_spawnattr_setflags(posix_spawnattr_t* __attr, short __flags) __INTRODUCED_IN(28);
int posix_spawnattr_getflags(const posix_spawnattr_t* __attr, short* __flags) __INTRODUCED_IN(28);

int posix_spawnattr_setpgroup(posix_spawnattr_t* __attr, pid_t __pgroup) __INTRODUCED_IN(28);
int posix_spawnattr_getpgroup(const posix_spawnattr_t* __attr, pid_t* __pgroup) __INTRODUCED_IN(28);

int posix_spawnattr_setsigmask(posix_spawnattr_t* __attr, const sigset_t* __mask) __INTRODUCED_IN(28);
int posix_spawnattr_setsigmask64(posix_spawnattr_t* __attr, const sigset64_t* __mask) __INTRODUCED_IN(28);
int posix_spawnattr_getsigmask(const posix_spawnattr_t* __attr, sigset_t* __mask) __INTRODUCED_IN(28);
int posix_spawnattr_getsigmask64(const posix_spawnattr_t* __attr, sigset64_t* __mask) __INTRODUCED_IN(28);

int posix_spawnattr_setsigdefault(posix_spawnattr_t* __attr, const sigset_t* __mask) __INTRODUCED_IN(28);
int posix_spawnattr_setsigdefault64(posix_spawnattr_t* __attr, const sigset64_t* __mask) __INTRODUCED_IN(28);
int posix_spawnattr_getsigdefault(const posix_spawnattr_t* __attr, sigset_t* __mask) __INTRODUCED_IN(28);
int posix_spawnattr_getsigdefault64(const posix_spawnattr_t* __attr, sigset64_t* __mask) __INTRODUCED_IN(28);

int posix_spawnattr_setschedparam(posix_spawnattr_t* __attr, const struct sched_param* __param) __INTRODUCED_IN(28);
int posix_spawnattr_getschedparam(const posix_spawnattr_t* __attr, struct sched_param* __param) __INTRODUCED_IN(28);

int posix_spawnattr_setschedpolicy(posix_spawnattr_t* __attr, int __policy) __INTRODUCED_IN(28);
int posix_spawnattr_getschedpolicy(const posix_spawnattr_t* __attr, int* __policy) __INTRODUCED_IN(28);

int posix_spawn_file_actions_init(posix_spawn_file_actions_t* __actions) __INTRODUCED_IN(28);
int posix_spawn_file_actions_destroy(posix_spawn_file_actions_t* __actions) __INTRODUCED_IN(28);

int posix_spawn_file_actions_addopen(posix_spawn_file_actions_t* __actions, int __fd, const char* __path, int __flags, mode_t __mode) __INTRODUCED_IN(28);
int posix_spawn_file_actions_addclose(posix_spawn_file_actions_t* __actions, int __fd) __INTRODUCED_IN(28);
int posix_spawn_file_actions_adddup2(posix_spawn_file_actions_t* __actions, int __fd, int __new_fd) __INTRODUCED_IN(28);
#endif /* __ANDROID_API__ >= 28 */


__END_DECLS

#endif

// End of embedded file



// These stay undefined forever
#undef posix_spawnattr_setsigmask64
#undef posix_spawnattr_getsigmask64
#undef posix_spawnattr_setsigdefault64
#undef posix_spawnattr_getsigdefault64

#undef posix_spawn
#undef posix_spawnp
#undef posix_spawnattr_init
#undef posix_spawnattr_destroy
#undef posix_spawnattr_setflags
#undef posix_spawnattr_getflags
#undef posix_spawnattr_setpgroup
#undef posix_spawnattr_getpgroup
#undef posix_spawnattr_setsigmask
#undef posix_spawnattr_getsigmask
#undef posix_spawnattr_setsigdefault
#undef posix_spawnattr_getsigdefault
#undef posix_spawnattr_setschedparam
#undef posix_spawnattr_getschedparam
#undef posix_spawnattr_setschedpolicy
#undef posix_spawnattr_getschedpolicy
#undef posix_spawn_file_actions_init
#undef posix_spawn_file_actions_destroy
#undef posix_spawn_file_actions_addopen
#undef posix_spawn_file_actions_addclose
#undef posix_spawn_file_actions_adddup2

#define posix_spawn backcompat_posix_spawn
#define posix_spawnp backcompat_posix_spawnp
#define posix_spawnattr_init backcompat_posix_spawnattr_init
#define posix_spawnattr_destroy backcompat_posix_spawnattr_destroy
#define posix_spawnattr_setflags backcompat_posix_spawnattr_setflags
#define posix_spawnattr_getflags backcompat_posix_spawnattr_getflags
#define posix_spawnattr_setpgroup backcompat_posix_spawnattr_setpgroup
#define posix_spawnattr_getpgroup backcompat_posix_spawnattr_getpgroup
#define posix_spawnattr_setsigmask backcompat_posix_spawnattr_setsigmask
#define posix_spawnattr_getsigmask backcompat_posix_spawnattr_getsigmask
#define posix_spawnattr_setsigdefault backcompat_posix_spawnattr_setsigdefault
#define posix_spawnattr_getsigdefault backcompat_posix_spawnattr_getsigdefault
#define posix_spawnattr_setschedparam backcompat_posix_spawnattr_setschedparam
#define posix_spawnattr_getschedparam backcompat_posix_spawnattr_getschedparam
#define posix_spawnattr_setschedpolicy backcompat_posix_spawnattr_setschedpolicy
#define posix_spawnattr_getschedpolicy backcompat_posix_spawnattr_getschedpolicy
#define posix_spawn_file_actions_init backcompat_posix_spawn_file_actions_init
#define posix_spawn_file_actions_destroy backcompat_posix_spawn_file_actions_destroy
#define posix_spawn_file_actions_addopen backcompat_posix_spawn_file_actions_addopen
#define posix_spawn_file_actions_addclose backcompat_posix_spawn_file_actions_addclose
#define posix_spawn_file_actions_adddup2 backcompat_posix_spawn_file_actions_adddup2

#ifndef __BEGIN_DECLS
#ifdef   __cplusplus
# define __BEGIN_DECLS  extern "C" {                                            
# define __END_DECLS }
#else
# define __BEGIN_DECLS
# define __END_DECLS
#endif
#endif

__BEGIN_DECLS

int posix_spawn(pid_t* pid, const char* path, const posix_spawn_file_actions_t* actions, const posix_spawnattr_t* attr, char* const argv[], char* const env[]);
int posix_spawnp(pid_t* pid, const char* file, const posix_spawn_file_actions_t* actions, const posix_spawnattr_t* attr, char* const argv[], char* const env[]);
int posix_spawnattr_init(posix_spawnattr_t* attr);
int posix_spawnattr_destroy(posix_spawnattr_t* attr);
int posix_spawnattr_setflags(posix_spawnattr_t* attr, short flags);
int posix_spawnattr_getflags(const posix_spawnattr_t* attr, short* flags);
int posix_spawnattr_setpgroup(posix_spawnattr_t* attr, pid_t pgroup);
int posix_spawnattr_getpgroup(const posix_spawnattr_t* attr, pid_t* pgroup);
int posix_spawnattr_setsigmask(posix_spawnattr_t* attr, const sigset_t* mask);
int posix_spawnattr_getsigmask(const posix_spawnattr_t* attr, sigset_t* mask);
int posix_spawnattr_setsigdefault(posix_spawnattr_t* attr, const sigset_t* mask);
int posix_spawnattr_getsigdefault(const posix_spawnattr_t* attr, sigset_t* mask);
int posix_spawnattr_setschedparam(posix_spawnattr_t* attr, const struct sched_param* param);
int posix_spawnattr_getschedparam(const posix_spawnattr_t* attr, struct sched_param* param);
int posix_spawnattr_setschedpolicy(posix_spawnattr_t* attr, int policy);
int posix_spawnattr_getschedpolicy(const posix_spawnattr_t* attr, int* policy);
int posix_spawn_file_actions_init(posix_spawn_file_actions_t* actions);
int posix_spawn_file_actions_destroy(posix_spawn_file_actions_t* actions);
int posix_spawn_file_actions_addopen(posix_spawn_file_actions_t* actions, int fd, const char* path, int flags, mode_t mode);
int posix_spawn_file_actions_addclose(posix_spawn_file_actions_t* actions, int fd);
int posix_spawn_file_actions_adddup2(posix_spawn_file_actions_t* actions, int fd, int new_fd);

#ifndef LIBBACKCOMPAT_BUILDING

#undef posix_spawn
#undef posix_spawnp
#undef posix_spawnattr_init
#undef posix_spawnattr_destroy
#undef posix_spawnattr_setflags
#undef posix_spawnattr_getflags
#undef posix_spawnattr_setpgroup
#undef posix_spawnattr_getpgroup
#undef posix_spawnattr_setsigmask
#undef posix_spawnattr_getsigmask
#undef posix_spawnattr_setsigdefault
#undef posix_spawnattr_getsigdefault
#undef posix_spawnattr_setschedparam
#undef posix_spawnattr_getschedparam
#undef posix_spawnattr_setschedpolicy
#undef posix_spawnattr_getschedpolicy
#undef posix_spawn_file_actions_init
#undef posix_spawn_file_actions_destroy
#undef posix_spawn_file_actions_addopen
#undef posix_spawn_file_actions_addclose
#undef posix_spawn_file_actions_adddup2

static __inline__ __attribute__((__always_inline__)) int posix_spawn(pid_t* pid, const char* path, const posix_spawn_file_actions_t* actions, const posix_spawnattr_t* attr, char* const argv[], char* const env[])
{
	return backcompat_posix_spawn(pid, path, actions, attr, argv, env);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawnp(pid_t* pid, const char* file, const posix_spawn_file_actions_t* actions, const posix_spawnattr_t* attr, char* const argv[], char* const env[])
{
	return backcompat_posix_spawnp(pid, file, actions, attr, argv, env);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawnattr_init(posix_spawnattr_t* attr)
{
	return backcompat_posix_spawnattr_init(attr);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawnattr_destroy(posix_spawnattr_t* attr)
{
	return backcompat_posix_spawnattr_destroy(attr);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawnattr_setflags(posix_spawnattr_t* attr, short flags)
{
	return backcompat_posix_spawnattr_setflags(attr, flags);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawnattr_getflags(const posix_spawnattr_t* attr, short* flags)
{
	return backcompat_posix_spawnattr_getflags(attr, flags);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawnattr_setpgroup(posix_spawnattr_t* attr, pid_t pgroup)
{
	return backcompat_posix_spawnattr_setpgroup(attr, pgroup);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawnattr_getpgroup(const posix_spawnattr_t* attr, pid_t* pgroup)
{
	return backcompat_posix_spawnattr_getpgroup(attr, pgroup);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawnattr_setsigmask(posix_spawnattr_t* attr, const sigset_t* mask)
{
	return backcompat_posix_spawnattr_setsigmask(attr, mask);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawnattr_getsigmask(const posix_spawnattr_t* attr, sigset_t* mask)
{
	return backcompat_posix_spawnattr_getsigmask(attr, mask);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawnattr_setsigdefault(posix_spawnattr_t* attr, const sigset_t* mask)
{
	return backcompat_posix_spawnattr_setsigdefault(attr, mask);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawnattr_getsigdefault(const posix_spawnattr_t* attr, sigset_t* mask)
{
	return backcompat_posix_spawnattr_getsigdefault(attr, mask);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawnattr_setschedparam(posix_spawnattr_t* attr, const struct sched_param* param)
{
	return backcompat_posix_spawnattr_setschedparam(attr, param);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawnattr_getschedparam(const posix_spawnattr_t* attr, struct sched_param* param)
{
	return backcompat_posix_spawnattr_getschedparam(attr, param);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawnattr_setschedpolicy(posix_spawnattr_t* attr, int policy)
{
	return backcompat_posix_spawnattr_setschedpolicy(attr, policy);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawnattr_getschedpolicy(const posix_spawnattr_t* attr, int* policy)
{
	return backcompat_posix_spawnattr_getschedpolicy(attr, policy);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawn_file_actions_init(posix_spawn_file_actions_t* actions)
{
	return backcompat_posix_spawn_file_actions_init(actions);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawn_file_actions_destroy(posix_spawn_file_actions_t* actions)
{
	return backcompat_posix_spawn_file_actions_destroy(actions);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawn_file_actions_addopen(posix_spawn_file_actions_t* actions, int fd, const char* path, int flags, mode_t mode)
{
	return backcompat_posix_spawn_file_actions_addopen(actions, fd, path, flags, mode);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawn_file_actions_addclose(posix_spawn_file_actions_t* actions, int fd)
{
	return backcompat_posix_spawn_file_actions_addclose(actions, fd);
}
static __inline__ __attribute__((__always_inline__)) int posix_spawn_file_actions_adddup2(posix_spawn_file_actions_t* actions, int fd, int new_fd)
{
	return backcompat_posix_spawn_file_actions_adddup2(actions, fd, new_fd);
}

#endif

__END_DECLS

#endif
#endif
